/*****************************************************************************

   VCE_MAX_676_7S_TESTERS

   TESTE PARA LEITURA DE TENSO MXIMA SUPORTADA EM TRANSISTORES MOSFET/BIPOLARES/IGBTS

   OBJETIVO: IDENTIFICAR TRANSISTORES FALSIFICADOS 

   COM DISPLAY DE 7 SEGUIMENTOS (3 DGITOS)

   AUTOR: CLUDIO LRIOS

   DATA: 24/11/2023

   ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG PICSOURCE

   USO DIDTICO APENAS

******************************************************************************/
#include <16F676.h>
#device adc=10
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOPROTECT, NOBROWNOUT, NOMCLR, NOCPD, NOPUT


//BYTE
#byte portc = 0x07
#byte porta = 0x05
#byte trisa = 0x85
#byte trisc = 0x87
#byte ioca  = 0x96 //interrupo de mudana de pinos individuais
//BIT

#bit   k2  = 0x7.4
#bit   k1  = 0x7.3
#bit   k0  = 0x7.2
#bit   zerar = 0x5.3 //boto 'zerar'
#bit   raif  = 0xb.0 //flag da interrupo de mudana dos pinos ra0 a ra5

#define VALOR_TEMPO_AP    20 //tempo entre converes
#define VALOR_FILTRO1     5   //valor da filtragem anti-oscilao
#define VALOR_FILTRO2     5   //valor da filtragem anti-oscilao
#define VALOR_TOL         7

unsigned int1 f_z=0;
unsigned int8  dig0,dig1,dig2,dig3,timer_ap=0,filtro1=VALOR_FILTRO1,filtro2=VALOR_FILTRO2,dly=0;
unsigned int16 valor, valor_save=0,valor_lido,valor_zero=0,vv;


//constantes para converso 7 seguimentos

//                         ABCDEFGP   SEGUIMENTOS
//                         76543210
int8 const NUM[] = {     0B11111100,//0
                         0B01100000,//1
                         0B11011010,//2
                         0B11110010,//3
                         0B01100110,//4
                         0B10110110,//5
                         0B10111110,//6
                         0B11100000,//7
                         0B11111110,//8
                         0B11110110 //9
                                        };



void env_7seg(unsigned int8 dado){

      dado=NUM[dado]; //converte para 7 seguimentos
      portc=0; //desliga todas sadas
      porta=0;
      if(dado&2)porta|=4;  //seta ra2 'g'
      if(dado&4)porta|=2;  //seta ra1 'f'
      if(dado&8)portc|=32; //seta rc5 'e'
      if(dado&16)porta|=16; //seta ra4 'd'
      if(dado&32)porta|=32; //seta ra5 'c'
      if(dado&64)portc|=2;  //seta rc1 'b'
      if(dado&128)portc|=1;  //seta rc0 'a'
}



void main() {

   setup_adc_ports(sAN0|VSS_VDD);
   setup_adc(ADC_CLOCK_INTERNAL);
   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   trisa=0b001001; //ra0=in adc pino 13
   trisc=0;
   ioca=0b001000;//interrupo pino 4 ra3

   dig3=0;
   dig2=0;
   dig1=0;
   dig0=0;
   
   vv=read_eeprom(1);
   vv=vv<<8;
   vv=vv+read_eeprom(0);
   valor_zero=vv;

   for(;;){

   //-------------------------------------
    trisa=0b001001; //ra0=in adc pino 13
    trisc=0;
    //-------------------------------------

    timer_ap++;
    if(timer_ap>VALOR_TEMPO_AP){
    set_adc_channel(0); //canal '0' pino 13
    valor_lido=read_adc(ADC_START_AND_READ);
    timer_ap=0;



    if(valor_lido>valor_save){
      filtro1--;
      if(!filtro1){
      if(valor_save<(valor_lido+VALOR_TOL)) valor_save=valor_lido;
      if(valor_save<valor_lido) valor_save++;
      filtro1=VALOR_FILTRO1;
      filtro2=VALOR_FILTRO2;
      }

    }

     if(valor_lido<valor_save){
      filtro2--;
      if(!filtro2){
      if(valor_save>(valor_lido+VALOR_TOL)) valor_save=valor_lido;
      if(valor_save>valor_lido) valor_save--;
      filtro2=VALOR_FILTRO2;
      filtro1=VALOR_FILTRO1;
      }

    }


    }
//--------------------------------------------------------
//MOSTRA VALOR LIDO

  //  if(zerar==1)valor_zero=valor_save;// testa se boto "zerar" est apertado

    if(zerar==1){
     dly++;
     if(dly==255)valor_zero=valor_save;
     vv=read_eeprom(1);
     vv=vv<<8;
     vv=vv+read_eeprom(0);
     if(valor_zero!=vv){
       write_eeprom(0,valor_zero);
       write_eeprom(1,valor_zero>>8);
     }
    }
    else{
    dly=0;
    }

    valor=valor_save;//pega leitura de tenso



    //corrige  zero
    if(valor>=valor_zero){
      valor= valor - valor_zero;
    } else {
      valor= valor_zero - valor;
    }


    dig3=0;  //zera registrador dos digitos
    dig2=0;
    dig1=0;
    dig0=0;

    while(valor>=1000){
      dig3++; valor-=1000;

    }
     while(valor>=100){
      dig2++; valor-=100;

    }
     while(valor>=10){
      dig1++; valor-=10;

    }
    dig0=valor;



    if(dig2){  //mostra somente se for diferente de '0'
    env_7seg(dig2);
    k2=1;
    delay_ms(2);
    }
    if(dig1 || dig2){  //mostra somente se for diferente de '0'
    env_7seg(dig1);
    k1=1;
    delay_ms(2);
    }
    env_7seg(dig0);
    k0=1;
    delay_ms(2);





   }//for

}//main
